@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 to 2007 by Jonathan Duddington                     *
+ *   Copyright (C) 2005 to 2011 by Jonathan Duddington                     *
  *   email: jonsd@users.sourceforge.net                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -47,7 +47,6 @@
 #include "wave.h"
 
 unsigned char *outbuf=NULL;
-extern espeak_VOICE voice_selected;
 
 espeak_EVENT *event_list=NULL;
 int event_list_ix=0;
@@ -59,6 +58,9 @@ static unsigned int my_unique_identifier=0;
 static void* my_user_data=NULL;
 static espeak_AUDIO_OUTPUT my_mode=AUDIO_OUTPUT_SYNCHRONOUS;
 static int synchronous_mode = 1;
+static int out_samplerate = 0;
+static int voice_samplerate = 22050;
+
 t_espeak_callback* synth_callback = NULL;
 int (* uri_callback)(int, const char *, const char *) = NULL;
 int (* phoneme_callback)(const char *) = NULL;
@@ -66,6 +68,13 @@ int (* phoneme_callback)(const char *) = NULL;
 char path_home[N_PATH_HOME];   // this is the espeak-data directory
 
 
+void WVoiceChanged(voice_t *wvoice)
+{//=================================
+// Voice change in wavegen
+	voice_samplerate = wvoice->samplerate;
+}
+
+
 #ifdef USE_ASYNC
 
 static int dispatch_audio(short* outbuf, int length, espeak_EVENT* event)
@@ -85,6 +94,32 @@ static int dispatch_audio(short* outbuf, int length, espeak_EVENT* event)
 	{
 	case AUDIO_OUTPUT_PLAYBACK:
 	{
+		int event_type=0;
+		if(event)
+		{
+			event_type = event->type;
+		}
+
+		if(event_type == espeakEVENT_SAMPLERATE)
+		{
+			voice_samplerate = event->id.number;
+
+			if(out_samplerate != voice_samplerate)
+			{
+				if(out_samplerate != 0)
+				{
+					// sound was previously open with a different sample rate
+					wave_close(my_audio);
+					sleep(1);
+				}
+				out_samplerate = voice_samplerate;
+				wave_init(voice_samplerate);
+				wave_set_callback_is_output_enabled( fifo_is_command_enabled);
+				my_audio = wave_open("alsa");
+				event_init();
+			}
+		}
+
 		if (outbuf && length && a_wave_can_be_played)
 		{
 			wave_write (my_audio, (char*)outbuf, 2*length);
@@ -173,7 +208,7 @@ static int create_events(short* outbuf, int length, espeak_EVENT* event, uint32_
 }
 
 
-int sync_espeak_terminated_msg( uint unique_identifier, void* user_data)
+int sync_espeak_terminated_msg( uint32_t unique_identifier, void* user_data)
 {//=====================================================================
 	ENTER("sync_espeak_terminated_msg");
 
@@ -220,17 +255,13 @@ static void select_output(espeak_AUDIO_OUTPUT output_type)
 	my_audio = NULL;
 	synchronous_mode = 1;
  	option_waveout = 1;   // inhibit portaudio callback from wavegen.cpp
+	out_samplerate = 0;
 
 	switch(my_mode)
 	{
 	case AUDIO_OUTPUT_PLAYBACK:
+		// wave_init() is now called just before the first wave_write()
 		synchronous_mode = 0;
-#ifdef USE_ASYNC
-		wave_init();
-		wave_set_callback_is_output_enabled( fifo_is_command_enabled);
-		my_audio = wave_open("alsa");
-		event_init();
-#endif
 		break;
 
 	case AUDIO_OUTPUT_RETRIEVAL:
@@ -336,8 +367,8 @@ static void init_path(const char *path)
 #endif
 }
 
-static int initialise(void)
-{//========================
+static int initialise(int control)
+{//===============================
 	int param;
 	int result;
 
@@ -348,14 +379,17 @@ static int initialise(void)
 		if(result == -1)
 		{
 			fprintf(stderr,"Failed to load espeak-data\n");
-			exit(1);
+			if((control & espeakINITIALIZE_DONT_EXIT) == 0)
+			{
+				exit(1);
+			}
 		}
 		else
 			fprintf(stderr,"Wrong version of espeak-data 0x%x (expects 0x%x) at %s\n",result,version_phdata,path_home);
 	}
 
-	memset(&voice_selected,0,sizeof(voice_selected));
-	SetVoiceStack(NULL);
+	memset(&current_voice_selected,0,sizeof(current_voice_selected));
+	SetVoiceStack(NULL, "");
 	SynthesizeInit();
 	InitNamedata();
 
@@ -494,8 +528,8 @@ static espeak_ERROR Synthesize(unsigned int unique_identifier, const void *text,
 				}
 			}
 		}
-    }
-  return(EE_OK);
+	}
+	return(EE_OK);
 }  //  end of Synthesize
 
 #ifdef DEBUG_ENABLED
@@ -509,9 +543,9 @@ static const char* label[] = {
 #endif
 
 
-void MarkerEvent(int type, unsigned int char_position, int value, unsigned char *out_ptr)
-{//======================================================================================
-	// type: 1=word, 2=sentence, 3=named mark, 4=play audio, 5=end
+void MarkerEvent(int type, unsigned int char_position, int value, int value2, unsigned char *out_ptr)
+{//==================================================================================================
+	// type: 1=word, 2=sentence, 3=named mark, 4=play audio, 5=end, 7=phoneme
 	ENTER("MarkerEvent");
 	espeak_EVENT *ep;
 	double time;
@@ -523,7 +557,7 @@ void MarkerEvent(int type, unsigned int char_position, int value, unsigned char
 	ep->type = (espeak_EVENT_TYPE)type;
 	ep->unique_identifier = my_unique_identifier;
 	ep->user_data = my_user_data;
-	ep->text_position = char_position & 0xffff;
+	ep->text_position = char_position & 0xffffff;
 	ep->length = char_position >> 24;
 	
 	time = (double(count_samples + mbrola_delay + (out_ptr - out_start)/2)*1000.0)/samplerate;
@@ -540,7 +574,17 @@ void MarkerEvent(int type, unsigned int char_position, int value, unsigned char
 	if((type == espeakEVENT_MARK) || (type == espeakEVENT_PLAY))
 		ep->id.name = &namedata[value];
 	else
+	if(type == espeakEVENT_PHONEME)
+	{
+		int *p;
+		p = (int *)(ep->id.string);
+		p[0] = value;
+		p[1] = value2;
+	}
+	else
+	{
 		ep->id.number = value;
+	}
 }  //  end of MarkerEvent
 
 
@@ -711,9 +755,14 @@ ENTER("espeak_Initialize");
 #endif
 	
 	init_path(path);
-	initialise();
+	initialise(options);
 	select_output(output_type);
-	
+
+	if(f_logespeak)
+	{
+		fprintf(f_logespeak,"INIT mode %d options 0x%x\n",output_type,options);
+	}
+
 	// buflength is in mS, allocate 2 bytes per sample
 	if(buf_length == 0)
 		buf_length = 200;
@@ -729,19 +778,21 @@ ENTER("espeak_Initialize");
 		return(EE_INTERNAL_ERROR);
 	
 	option_phonemes = 0;
-	option_phoneme_events = (options & 1);
+	option_mbrola_phonemes = 0;
+	option_phoneme_events = (options & (espeakINITIALIZE_PHONEME_EVENTS | espeakINITIALIZE_PHONEME_IPA));
 
-	SetVoiceByName("default");
+	VoiceReset(0);
+//	SetVoiceByName("default");
 	
 	for(param=0; param<N_SPEECH_PARAM; param++)
 		param_stack[0].parameter[param] = param_defaults[param];
 	
-	SetParameter(espeakRATE,170,0);
+	SetParameter(espeakRATE,175,0);
 	SetParameter(espeakVOLUME,100,0);
 	SetParameter(espeakCAPITALS,option_capitals,0);
 	SetParameter(espeakPUNCTUATION,option_punctuation,0);
 	SetParameter(espeakWORDGAP,0,0);
-	DoVoiceChange(voice);
+//	DoVoiceChange(voice);
 	
 #ifdef USE_ASYNC
 	fifo_init();
@@ -763,6 +814,12 @@ ESPEAK_API espeak_ERROR espeak_Synth(const void *text, size_t size,
 	SHOW("espeak_Synth > position=%d, position_type=%d, end_position=%d, flags=%d, user_data=0x%x, text=%s\n", position, position_type, end_position, flags, user_data, text);
 #endif
 
+	if(f_logespeak)
+	{
+		fprintf(f_logespeak,"\nSYNTH posn %d %d %d flags 0x%x\n%s\n",position,end_position,position_type,flags, (const char *)text);
+		fflush(f_logespeak);
+	}
+
 	espeak_ERROR a_error=EE_INTERNAL_ERROR;
 	static unsigned int temp_identifier;
 
@@ -825,6 +882,12 @@ ESPEAK_API espeak_ERROR espeak_Synth_Mark(const void *text, size_t size,
 	espeak_ERROR a_error=EE_OK;
 	static unsigned int temp_identifier;
 
+	if(f_logespeak)
+	{
+		fprintf(f_logespeak,"\nSYNTH MARK %s posn %d flags 0x%x\n%s\n",index_mark,end_position,flags, (const char *)text);
+	}
+
+
 	if (unique_identifier == NULL)
 	{
 		unique_identifier = &temp_identifier;
@@ -875,6 +938,11 @@ ESPEAK_API espeak_ERROR espeak_Key(const char *key)
 	ENTER("espeak_Key");
 	// symbolic name, symbolicname_character  - is there a system resource of symbolicnames per language
 
+	if(f_logespeak)
+	{
+		fprintf(f_logespeak,"\nKEY %s\n",key);
+	}
+
 	espeak_ERROR a_error = EE_OK;
 
 	if(synchronous_mode)
@@ -884,7 +952,7 @@ ESPEAK_API espeak_ERROR espeak_Key(const char *key)
 	}
 
 #ifdef USE_ASYNC
-	t_espeak_command* c = create_espeak_key( key);
+	t_espeak_command* c = create_espeak_key( key, NULL);
 	a_error = fifo_add_command(c);
 	if (a_error != EE_OK)
 	{
@@ -901,6 +969,11 @@ ESPEAK_API espeak_ERROR espeak_Char(wchar_t character)
   ENTER("espeak_Char");
   // is there a system resource of character names per language?
 
+	if(f_logespeak)
+	{
+		fprintf(f_logespeak,"\nCHAR U+%x\n",character);
+	}
+
 #ifdef USE_ASYNC
 	espeak_ERROR a_error;
 
@@ -910,7 +983,7 @@ ESPEAK_API espeak_ERROR espeak_Char(wchar_t character)
 		return(EE_OK);
 	}
 
-	t_espeak_command* c = create_espeak_char( character);
+	t_espeak_command* c = create_espeak_char( character, NULL);
 	a_error = fifo_add_command(c);
 	if (a_error != EE_OK)
 	{
@@ -997,6 +1070,10 @@ ESPEAK_API espeak_ERROR espeak_SetParameter(espeak_PARAMETER parameter, int valu
 {//=============================================================================================
   ENTER("espeak_SetParameter");
 
+	if(f_logespeak)
+	{
+		fprintf(f_logespeak,"SETPARAM %d %d %d\n",parameter,value,relative);
+	}
 #ifdef USE_ASYNC
 	espeak_ERROR a_error;
 
@@ -1053,11 +1130,15 @@ ESPEAK_API void espeak_SetPhonemeTrace(int value, FILE *stream)
 {//============================================================
 	ENTER("espeak_SetPhonemes");
 	/* Controls the output of phoneme symbols for the text
-		value=0  No phoneme output (default)
-		value=1  Output the translated phoneme symbols for the text
-		value=2  as (1), but also output a trace of how the translation was done (matching rules and list entries)
+		bits 0-3:
+		 value=0  No phoneme output (default)
+		 value=1  Output the translated phoneme symbols for the text
+		 value=2  as (1), but also output a trace of how the translation was done (matching rules and list entries)
+		 value=3  as (1), but produces IPA phoneme names rather than ascii 
+		bit 4:   produce mbrola pho data
 	*/
-	option_phonemes = value;
+	option_phonemes = value & 3;
+	option_mbrola_phonemes = value & 16;
 	f_trans = stream;
 	if(stream == NULL)
 		f_trans = stderr;
@@ -1119,6 +1200,7 @@ ESPEAK_API espeak_ERROR espeak_Synchronize(void)
 
 
 extern void FreePhData(void);
+extern void FreeVoiceList(void);
 
 ESPEAK_API espeak_ERROR espeak_Terminate(void)
 {//===========================================
@@ -1132,6 +1214,7 @@ ESPEAK_API espeak_ERROR espeak_Terminate(void)
 	{
 		wave_close(my_audio);
 		wave_terminate();
+		out_samplerate = 0;
 	}
 
 #endif
@@ -1140,12 +1223,23 @@ ESPEAK_API espeak_ERROR espeak_Terminate(void)
 	Free(outbuf);
 	outbuf = NULL;
 	FreePhData();
+	FreeVoiceList();
+
+	if(f_logespeak)
+	{
+		fclose(f_logespeak);
+		f_logespeak = NULL;
+	}
 
 	return EE_OK;
 }   //  end of espeak_Terminate
 
-ESPEAK_API const char *espeak_Info(void *)
-{//=======================================
+ESPEAK_API const char *espeak_Info(const char **ptr)
+{//=================================================
+	if(ptr != NULL)
+	{
+		*ptr = path_home;
+	}
 	return(version_string);
 }
 
